# Debugging Sandbox

# Code throws this error:
# Error in if (all(aDF[a, ] == aRow[1, ])) { : 
#     missing value where TRUE/FALSE needed

# Start with a clean environment
rm(list = ls())

library("XML")
library("here")

xmlURL <- "http://artificium.us/lessons/06.r/l-6-191-debugging/journals.xml"

# Reading the XML file and parse into DOM
xmlDOM <- xmlParse(xmlURL)

# Get the root node of the DOM tree
r <- xmlRoot(xmlDOM)

# Get number of children of root (number of pubmed articles)
numPubArticles <- xmlSize(r)

## Create main data frames to hold data
PubmedArticle.df <- data.frame(Pmid = vector(mode = "integer", length = numPubArticles),
                               Journal = vector(mode = "integer", length = numPubArticles),
                               ArticleTitle = vector(mode = "character", length = numPubArticles),
                               Language = vector(mode = "character", length = numPubArticles),
                               stringsAsFactors = F)


# We do not know the number of <Journal> nodes we have (we cannot have duplicate journals), so we cannot pre-allocate the memory.
Journal.df <- data.frame(JournalId = integer(),
                         IssnType = character(),
                         Issn = character(),
                         CitedMedium = character(),
                         Volume = integer(),
                         Issue = integer(),
                         Year = integer(),
                         Month = character(),
                         Day = integer(),
                         Title = character(),
                         ISOAbbreviation = character())

# We do not know the number of <Author> nodes we have (we cannot have duplicate authors), so we cannot pre-allocate the memory.
Author.df <- data.frame(LastName = character(),
                        ForeName = character(),
                        Initials = character())

# We do not know the number of of <AuthorList> nodes we have, so we cannot pre-allocate the memory.
Authorship.df <- data.frame(Pmid = integer(),
                            IdAuthors = integer())

# Parse journal function
parseJournal <- function(aJournalNode) {
  newJournal.df <- data.frame(issnType = character(),
                              issn = character(),
                              citedMedium = character(),
                              volume = integer(),
                              issue = integer(),
                              year = integer(),
                              month = character(),
                              day = integer(),
                              title = character(),
                              iSOAbbreviation = character())
  
  issn1 <- aJournalNode[[1]]
  newJournal.df[1,2] <- xmlValue(issn1)
  it <- xmlAttrs(issn1)
  issnType1 <- as.character(it)
  newJournal.df[1,1] <- issnType1
  
  journalIssue <- aJournalNode[[2]]
  cm <- xmlAttrs(journalIssue)
  citedMedium1 <- as.character(cm)
  newJournal.df[1,3] <- citedMedium1
  
  volume1 <- journalIssue[[1]]
  issue1 <- journalIssue[[2]]
  newJournal.df[1,4] <- xmlValue(volume1)
  newJournal.df[1,5] <- xmlValue(issue1)
  
  # I have decided to make my dates separate year, month, day columns
  # In the R data frame year and day will be numeric and month will be character
  # In SQL I am going to use a month lookup table
  pubDate <- journalIssue[[3]]
  year1 <- pubDate[[1]]
  newJournal.df[1,6] <- xmlValue(year1)
  month1 <- pubDate[[2]]
  newJournal.df[1,7] <- xmlValue(month1)
  
  # <Day> is optional, so if not present set to 0
  # If it is present add it to the data frame
  day1 <- pubDate[[3]]
  if (!is.null(day1)) {
    newJournal.df[1,8] <- xmlValue(day1)
  } else
    newJournal.df[1,8] <- 0

  title1 <- aJournalNode[[3]]
  isoAbbrev1 <- aJournalNode[[4]]
  newJournal.df[1,9] <- xmlValue(title1)
  newJournal.df[1,10] <- xmlValue(isoAbbrev1)
  
  return (newJournal.df)
}

# Parse author function
parseAuthor <- function(anAuthorNode) {
  newAuthors.df <- data.frame(lastName = character(),
                              foreName = character(),
                              initials = character())
  n <- xmlSize(anAuthorNode)
  
  # Extract each of the <Author> nodes under <AuthorList>
  for (m in 1:n) {
    anAuthor <- anAuthorNode[[m]]
    
    # Extract first child nodes that are always present
    lastName <- xmlValue(anAuthor[[1]])
    foreName <- xmlValue(anAuthor[[2]])
    initials <- xmlValue(anAuthor[[3]])
    
    newAuthors.df[m,1] <- lastName
    newAuthors.df[m,2] <- foreName
    newAuthors.df[m,3] <- initials
  }
  return(newAuthors.df)
}

# Row exists function
rowExists <- function (aRow, aDF) {
  # Check if that address is already in the data frame
  n <- nrow(aDF)
  c <- ncol(aDF)
  
  if (n == 0) {
    # Data frame is empty, so can't exist
    return(0)
  }
  
  for (a in 1:n) {
    # Check if all columns match for a row; ignore the aID column
    
    if (all(aDF[a,] == aRow[1,])) {
      # Found a match; return it's ID
      return(a)
    }
  }
  
  # None matched
  return(0)
}

# Parse all the data in the XML file
# Iterate over the first-level children elements off the root:
# the <PubmedArticle> elements

for (i in 1:numPubArticles) {
  
  # Get next <PubMedArticle> node
  aPubmedArticle <- r[[i]]
  
  # Add the PMID to the PubmedArticle.df -- assigning PMID as I process each article as opposed to using the PMID from the XML file
  PubmedArticle.df$Pmid[i] <- i
  
  # Get <Article> node
  anArticle <- aPubmedArticle[[1]]
  
  # Get <Journal> node and parse that node for other attributes
  aJournal <- anArticle[[1]]
  journal <- parseJournal(aJournal)
  pk.journal <- rowExists(journal, Journal.df[,2:ncol(Journal.df)])
  
  if (pk.journal == 0)
  {
    # Does not exist, so add
    pk.journal <- nrow(Journal.df) + 1
    Journal.df[pk.journal,2:ncol(Journal.df)] <- journal[1,]
    Journal.df[pk.journal,1] <- pk.journal
  }
  
  # It does not matter if the article was already added or not, we still need to
  # add the Journal to the PubmedArticle.df
  PubmedArticle.df$Journal[i] <- pk.journal
  
  # Add the Article Title and Language to the PubmedArticle.df
  anArticleTitle <- anArticle[[2]]
  aLanguage <- anArticle[[4]]
  PubmedArticle.df$ArticleTitle[i] <- xmlValue(anArticleTitle)
  PubmedArticle.df$Language[i] <- xmlValue(aLanguage)
  
  # Get the <AuthorList> CompleteYN attribute
  authorList <- anArticle[[3]]
  
  # Process the set of Authors into a separate data frame
  pubmedAuthors <- parseAuthor(authorList)
  
  for (n in 1:nrow(pubmedAuthors)) {
    # Check if author already exists
    pk.author <- rowExists(pubmedAuthors[n,2:ncol(Author.df)], Author.df[,2:ncol(Author.df)])
    
    # No matter if the author already exists in the Author.df, we have to add it to the Authorship.df to keep track of what author wrote what Pubmed article
    k <- nrow(Authorship.df) + 1
    Authorship.df[k,1] <- i
    
    if (pk.author == 0) {
      # Does not exist, so add
      j <- nrow(Author.df) + 1
      Author.df[j,] <- pubmedAuthors[n,]
      Authorship.df[k,2] <- j
    } else {
      # Does exist, so do not add
      Authorship.df[k,2] <- pk.author
    }
  }
}

# Cast certain attributes to be of type numeric
Journal.df$Volume <- as.numeric(Journal.df$Volume)
Journal.df$Issue <- as.numeric(Journal.df$Issue)
Journal.df$Year <- as.numeric(Journal.df$Year)
Journal.df$Day <- as.numeric(Journal.df$Day)
